/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates the idle mode.
 *
 *    Observe that at least the interrupt rountines must be compiled in
 *    ARM code, as opposed to THUMB code. All code is compiled in ARM code
 *    for simplicity.
 *
 *    Also observe that the startup code framwork must handle the VIC and
 *    dispatch the correct interrupt ISRs. This can be done with Embedded
 *    Artists' startup framework by setting the define: IRQ_HANDLER to 1
 *    (meaning: Load vector directly from VIC, i.e., LDR PC,[PC,#-0xFF0]).
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU16 delayInMs);
static void initTimer0(tU16 delayInMs);
static void timer0ISR(void) __attribute__ ((interrupt));


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Actual ISR that is called whenever Timer #0 generated an interrupt.
 *
 ****************************************************************************/
static void
timer0ISR(void)
{
	//reset interrupt flag
	T0IR = 0x01;

  //inform VIC that ISR is done
  VICVectAddr = 0x00;
}   


/*****************************************************************************
 *
 * Description:
 *    Initialze timer #0 to trig interrupt after given time.
 *
 ****************************************************************************/
static void
initTimer0(tU16 delayInMs)
{
  //initialize VIC for Timer0 interrupts
  VICIntSelect &= ~0x10;           //Timer0 interrupt is assigned to IRQ (not FIQ)
  VICVectAddr4  = (tU32)timer0ISR; //register ISR address
  VICVectCntl4  = 0x24;            //enable vector interrupt for timer0
  VICIntEnable  = 0x10;            //enable timer0 interrupt
  
  //initialize and start Timer #0
  T0TCR = 0x00000002;                           //disable and reset Timer0
  T0PC  = 0x00000000;                           //no prescale of clock
  T0MR0 = delayInMs *                           //calculate no of timer ticks
         ((CRYSTAL_FREQUENCY * PLL_FACTOR) / (1000 * VPBDIV_FACTOR));
  T0IR  = 0x000000ff;                           //reset all flags before enable IRQs
  T0MCR = 0x00000003;                           //reset counter and generate IRQ on MR0 match
  T0TCR = 0x00000001;                           //start Timer0
}


/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (1000 * VPBDIV_FACTOR));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}

  
/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2005 Embedded Artists AB");
  printf("\n*");
  printf("\n* Welcome to this program that demonstrates the idle mode");
  printf("\n*");
  printf("\n*********************************************************");

  while(1)
  {
  	tU8 i;
  	
    printf("\n\nEntering idle mode in 5 seconds");
#if (USE_NEWLIB == 1)
    fflush(stdout);
#endif

    for(i=0; i<20; i++)
    {
    	//wait 250 ms
      delayMs(250);
      printf(".");
    }
    printf("\nTimer #0 will wake me up after 3 seconds - now entering idle mode ");
#if (USE_NEWLIB == 1)
    fflush(stdout);
#endif
    
    //initialize timer #0 interrupt (trig after 3 seconds)
    initTimer0(3000);
  
    PCONP = 0x0000000e;   //turn off all peripherals, except timer #0 & #1 and uart #0
                          //= 0x0000080e, if external memory controller is used
    PCON  = 0x00000001;   //enter idle mode    

    //idle mode entered...

    printf("\n\nNow I'm awake again!!!");
  }

  return 0;
}
